import datetime
import logging
import os
import sys
import time
import unittest
from xml.etree import ElementTree as ET

from nose2 import events, loader, result, session, tools
from nose2._vendor import six
from nose2.plugins import junitxml, logcapture
from nose2.plugins.loader import functions, generators, parameters, testcases
from nose2.tests._common import TestCase


def _fromisoformat(date_str):
    # use fromisoformat when it is available, but failover to strptime for python2 and
    # older python3 versions
    if hasattr(datetime.datetime, "fromisoformat"):
        return datetime.datetime.fromisoformat(date_str)
    return datetime.datetime.strptime(date_str, "%Y-%m-%dT%H:%M:%S.%f")


class TestJunitXmlPlugin(TestCase):
    _RUN_IN_TEMP = True

    BAD_FOR_XML_U = six.u("A\x07 B\x0B C\x10 D\uD900 " "E\uFFFE F\x80 G\x90 H\uFDDD")
    # UTF-8 string with double null (invalid)
    BAD_FOR_XML_B = six.b(
        "A\x07 B\x0b C\x10 D\xed\xa4\x80 "
        "E\xef\xbf\xbe F\xc2\x80 G\xc2\x90 H\xef\xb7\x9d "
        "\x00\x00"
    )

    # "byte" strings in PY2 and unicode in py3 works as expected will
    # will translate surrogates into UTF-16 characters  so BAD_FOR_XML_U
    # should have 8 letters follows by 0xFFFD, but only 4 when keeping
    # the discouraged/restricted ranges. Respectively:
    # "A\uFFFD B\uFFFD C\uFFFD D\uFFFD E\uFFFD F\uFFFD G\uFFFD H\uFFFD"
    # "A\uFFFD B\uFFFD C\uFFFD D\uFFFD E\uFFFD F\x80 G\x90 H\uFDDD"
    #
    # In Python 2 Invalid ascii characters seem to get escaped out as part
    # of tracebace.format_traceback so full and partial replacements are:
    # "A\uFFFD B\uFFFD C\uFFFD D\\\\ud900 E\\\\ufffe F\\\\x80 G\\\\x90 H\\\\ufddd"
    # "A\uFFFD B\uFFFD C\uFFFD D\\\\ud900 E\\\\ufffe F\\\\x80 G\\\\x90 H\\\\ufddd"
    #
    # Byte strings in py3 as errors are replaced by their representation string
    # So these will be safe and not have any replacements
    # "b'A\\x07 B\\x0b C\\x10 D\\xed\\xa4\\x80 E\\xef\\xbf\\xbe F\\xc2\\x80
    # G\\xc2\\x90 H\\xef\\xb7\\x9d \\x00\\x00"

    if sys.maxunicode <= 0xFFFF:
        EXPECTED_RE = six.u("^[\x09\x0A\x0D\x20\x21-\uD7FF\uE000-\uFFFD]*$")
        EXPECTED_RE_SAFE = six.u(
            "^[\x09\x0A\x0D\x20\x21-\x7E\x85" "\xA0-\uD7FF\uE000-\uFDCF\uFDF0-\uFFFD]*$"
        )
    else:
        EXPECTED_RE = six.u(
            "^[\x09\x0A\x0D\x20\x21-\uD7FF\uE000-\uFFFD" "\u10000-\u10FFFF]*$"
        )
        EXPECTED_RE_SAFE = six.u(
            "^[\x09\x0A\x0D\x20\x21-\x7E\x85"
            "\xA0-\uD7FF\uE000-\uFDCF\uFDF0-\uFFFD"
            "\u10000-\u1FFFD\u20000-\u2FFFD"
            "\u30000-\u3FFFD\u40000-\u4FFFD"
            "\u50000-\u5FFFD\u60000-\u6FFFD"
            "\u70000-\u7FFFD\u80000-\u8FFFD"
            "\u90000-\u8FFFD\uA0000-\uAFFFD"
            "\uB0000-\uBFFFD\uC0000-\uCFFFD"
            "\uD0000-\uDFFFD\uE0000-\uEFFFD"
            "\uF0000-\uFFFFD\u100000-\u10FFFD]*$"
        )

    def setUp(self):
        super(TestJunitXmlPlugin, self).setUp()
        self.session = session.Session()
        self.loader = loader.PluggableTestLoader(self.session)
        self.result = result.PluggableTestResult(self.session)
        self.plugin = junitxml.JUnitXmlReporter(session=self.session)
        self.plugin.register()

        # Python 2.7 needs this
        # assertRegexpMatches() was renamed to assertRegex() in 3.2
        if not hasattr(self, "assertRegex"):
            self.assertRegex = self.assertRegexpMatches

        class Test(unittest.TestCase):
            def test(self):
                pass

            def test_chdir(self):
                TEMP_SUBFOLDER = "test_chdir"

                os.mkdir(TEMP_SUBFOLDER)
                os.chdir(TEMP_SUBFOLDER)

            def test_fail(self):
                assert False

            def test_err(self):
                1 / 0

            def test_skip(self):
                self.skipTest("skip")

            def test_skip_no_reason(self):
                self.skipTest("")

            def test_bad_xml(self):
                raise RuntimeError(TestJunitXmlPlugin.BAD_FOR_XML_U)

            def test_bad_xml_b(self):
                raise RuntimeError(TestJunitXmlPlugin.BAD_FOR_XML_B)

            def test_gen(self):
                def check(a, b):
                    # workaround:
                    # for the test which ensures that the timestamp increases between
                    # generator test runs, insert a very small sleep
                    #
                    # on py3.10 on Windows, we have observed both of these tests getting
                    # the same timestamp
                    time.sleep(0.001)
                    self.assertEqual(a, b)

                yield check, 1, 1
                yield check, 1, 2

            @tools.params(1, 2, 3)
            def test_params(self, p):
                self.assertEqual(p, 2)

            def test_with_log(self):
                logging.info("log message")

        self.case = Test

    def test_success_added_to_xml(self):
        test = self.case("test")
        test(self.result)
        self.assertEqual(self.plugin.numtests, 1)
        self.assertEqual(len(self.plugin.tree.findall("testcase")), 1)

    def test_failure_includes_traceback(self):
        test = self.case("test_fail")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        failure = case.find("failure")
        assert failure is not None
        assert "Traceback" in failure.text

    def test_error_bad_xml(self):
        self.plugin.keep_restricted = False
        test = self.case("test_bad_xml")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        self.assertRegex(error.text, self.EXPECTED_RE_SAFE)

    def test_error_bad_xml_keep(self):
        self.plugin.keep_restricted = True
        test = self.case("test_bad_xml")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        self.assertRegex(error.text, self.EXPECTED_RE)

    def test_error_bad_xml_b(self):
        self.plugin.keep_restricted = False
        test = self.case("test_bad_xml_b")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        assert error is not None
        self.assertRegex(error.text, self.EXPECTED_RE_SAFE)

    def test_error_bad_xml_b_keep(self):
        self.plugin.keep_restricted = True
        test = self.case("test_bad_xml_b")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        assert error is not None
        self.assertRegex(error.text, self.EXPECTED_RE)

    def test_error_includes_traceback(self):
        test = self.case("test_err")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        error = case.find("error")
        assert error is not None
        assert "Traceback" in error.text

    def test_skip_includes_skipped(self):
        test = self.case("test_skip")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        skip = case.find("skipped")
        assert skip is not None
        self.assertEqual(skip.get("message"), "test skipped: skip")
        self.assertEqual(skip.text, "skip")

    def test_skip_includes_skipped_no_reason(self):
        test = self.case("test_skip_no_reason")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        skip = case.find("skipped")
        assert skip is not None
        self.assertIsNone(skip.get("message"))
        self.assertIsNone(skip.text)

    def test_generator_timestamp_increases(self):
        gen = generators.Generators(session=self.session)
        gen.register()
        event = events.LoadFromTestCaseEvent(self.loader, self.case)
        self.session.hooks.loadTestsFromTestCase(event)
        cases = event.extraTests
        for case in cases:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 2)
        test1_timestamp_str = xml[0].get("timestamp")
        test1_timestamp = _fromisoformat(test1_timestamp_str)
        test2_timestamp_str = xml[1].get("timestamp")
        test2_timestamp = _fromisoformat(test2_timestamp_str)
        self.assertGreater(test2_timestamp, test1_timestamp)

    def test_generator_test_name_correct(self):
        gen = generators.Generators(session=self.session)
        gen.register()
        event = events.LoadFromTestCaseEvent(self.loader, self.case)
        self.session.hooks.loadTestsFromTestCase(event)
        cases = event.extraTests
        for case in cases:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 2)
        self.assertEqual(xml[0].get("name"), "test_gen:1")
        self.assertEqual(xml[1].get("name"), "test_gen:2")

    def test_generator_test_full_name_correct(self):
        gen = generators.Generators(session=self.session)
        gen.register()
        self.plugin.test_fullname = True
        event = events.LoadFromTestCaseEvent(self.loader, self.case)
        self.session.hooks.loadTestsFromTestCase(event)
        cases = event.extraTests
        for case in cases:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 2)
        self.assertEqual(xml[0].get("name"), "test_gen:1 (1, 1)")
        self.assertEqual(xml[1].get("name"), "test_gen:2 (1, 2)")

    def test_function_classname_is_module(self):
        fun = functions.Functions(session=self.session)
        fun.register()

        def test_func():
            pass

        cases = fun._createTests(test_func)
        self.assertEqual(len(cases), 1)
        cases[0](self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 1)
        self.assertTrue(xml[0].get("classname").endswith("test_junitxml"))

    def test_params_test_name_correct(self):
        # param test loading is a bit more complex than generator
        # loading. XXX -- can these be reconciled so they both
        # support exclude and also both support loadTestsFromTestCase?
        plug1 = parameters.Parameters(session=self.session)
        plug1.register()
        plug2 = testcases.TestCaseLoader(session=self.session)
        plug2.register()
        # need module to fire top-level event

        class Mod(object):
            pass

        m = Mod()
        m.Test = self.case
        event = events.LoadFromModuleEvent(self.loader, m)
        self.session.hooks.loadTestsFromModule(event)
        for case in event.extraTests:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 13)
        params = [x for x in xml if x.get("name").startswith("test_params")]
        self.assertEqual(len(params), 3)
        self.assertEqual(params[0].get("name"), "test_params:1")
        self.assertEqual(params[1].get("name"), "test_params:2")
        self.assertEqual(params[2].get("name"), "test_params:3")

    def test_params_test_full_name_correct(self):
        plug1 = parameters.Parameters(session=self.session)
        plug1.register()
        plug2 = testcases.TestCaseLoader(session=self.session)
        plug2.register()
        # need module to fire top-level event

        class Mod(object):
            pass

        m = Mod()
        m.Test = self.case
        event = events.LoadFromModuleEvent(self.loader, m)
        self.plugin.test_fullname = True
        self.session.hooks.loadTestsFromModule(event)
        for case in event.extraTests:
            case(self.result)
        xml = self.plugin.tree.findall("testcase")
        self.assertEqual(len(xml), 13)
        params = [x for x in xml if x.get("name").startswith("test_params")]
        self.assertEqual(len(params), 3)
        self.assertEqual(params[0].get("name"), "test_params:1 (1)")
        self.assertEqual(params[1].get("name"), "test_params:2 (2)")
        self.assertEqual(params[2].get("name"), "test_params:3 (3)")

    def test_writes_xml_file_at_end(self):
        test = self.case("test")
        test(self.result)
        event = events.StopTestRunEvent(None, self.result, 1, 1)
        self.plugin.stopTestRun(event)
        with open(self.plugin.path, "r") as fh:
            tree = ET.parse(fh).getroot()
        self.assertEqual(len(tree.findall("testcase")), 1)
        case = tree.find("testcase")
        assert "time" in case.attrib
        assert "timestamp" in case.attrib
        assert "classname" in case.attrib
        self.assertEqual(case.get("name"), "test")
        self.assertEqual(tree.get("errors"), "0")
        self.assertEqual(tree.get("failures"), "0")
        self.assertEqual(tree.get("skipped"), "0")
        self.assertEqual(tree.get("tests"), "1")
        assert "time" in tree.attrib

    def test_xml_file_path_is_not_affected_by_chdir_in_test(self):
        initial_dir = os.path.realpath(os.getcwd())
        test = self.case("test_chdir")
        test(self.result)
        plugin_dir = os.path.realpath(os.path.dirname(self.plugin.path))
        assert initial_dir == plugin_dir, self.plugin.path

    def test_xml_contains_empty_system_out_without_logcapture(self):
        test = self.case("test_with_log")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        system_out = case.find("system-out")
        assert system_out is not None
        assert not system_out.text

    def test_xml_contains_log_message_in_system_out_with_logcapture(self):
        self.logcapture_plugin = logcapture.LogCapture(session=self.session)
        self.logcapture_plugin.register()

        test = self.case("test_with_log")
        test(self.result)
        case = self.plugin.tree.find("testcase")
        system_out = case.find("system-out")
        assert system_out is not None
        assert "log message" in system_out.text
        assert "INFO" in system_out.text



import multiprocessing
import sys
import threading
import time
import unittest
from multiprocessing import connection

from nose2 import session
from nose2._vendor import six
from nose2.plugins import buffer
from nose2.plugins.loader import discovery, testcases
from nose2.plugins.mp import MultiProcess, procserver
from nose2.tests._common import Conn, FunctionalTestCase, _method_name, support_file


class TestMpPlugin(FunctionalTestCase):
    def setUp(self):
        super(TestMpPlugin, self).setUp()
        self.session = session.Session()
        self.plugin = MultiProcess(session=self.session)
        self.plugin.testRunTimeout = 2

    def test_flatten_without_fixtures(self):
        sys.path.append(support_file("scenario/slow"))
        import test_slow as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.TestSlow("test_ok"))
        suite.addTest(mod.TestSlow("test_fail"))
        suite.addTest(mod.TestSlow("test_err"))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(len(flat), 3)

    def test_flatten_nested_suites(self):
        sys.path.append(support_file("scenario/slow"))
        import test_slow as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.TestSlow("test_ok"))
        suite.addTest(mod.TestSlow("test_fail"))
        suite.addTest(mod.TestSlow("test_err"))

        suite2 = unittest.TestSuite()
        suite2.addTest(suite)

        flat = list(self.plugin._flatten(suite2))
        self.assertEqual(len(flat), 3)

    def test_flatten_respects_module_fixtures(self):
        sys.path.append(support_file("scenario/module_fixtures"))
        import test_mf_testcase as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.Test("test_1"))
        suite.addTest(mod.Test("test_2"))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(flat, ["test_mf_testcase"])

    def test_flatten_respects_class_fixtures(self):
        sys.path.append(support_file("scenario/class_fixtures"))
        import test_cf_testcase as mod

        suite = unittest.TestSuite()
        suite.addTest(mod.Test("test_1"))
        suite.addTest(mod.Test("test_2"))
        suite.addTest(mod.Test2("test_1"))
        suite.addTest(mod.Test2("test_2"))
        suite.addTest(mod.Test3("test_3"))

        flat = list(self.plugin._flatten(suite))
        self.assertEqual(
            flat,
            [
                "test_cf_testcase.Test2.test_1",
                "test_cf_testcase.Test2.test_2",
                "test_cf_testcase.Test",
                "test_cf_testcase.Test3",
            ],
        )

    def test_conn_prep(self):
        self.plugin.bind_host = None
        (parent_conn, child_conn) = self.plugin._prepConns()
        (parent_pipe, child_pipe) = multiprocessing.Pipe()
        self.assertIsInstance(parent_conn, type(parent_pipe))
        self.assertIsInstance(child_conn, type(child_pipe))

        self.plugin.bind_host = "127.0.0.1"
        self.plugin.bind_port = 0
        (parent_conn, child_conn) = self.plugin._prepConns()
        self.assertIsInstance(parent_conn, connection.Listener)
        self.assertIsInstance(child_conn, tuple)
        self.assertEqual(parent_conn.address, child_conn[:2])

    def test_conn_accept(self):
        (parent_conn, child_conn) = multiprocessing.Pipe()
        self.assertEqual(self.plugin._acceptConns(parent_conn), parent_conn)

        listener = connection.Listener(("127.0.0.1", 0))
        with self.assertRaises(RuntimeError):
            self.plugin._acceptConns(listener)

        def fake_client(address):
            client = connection.Client(address)
            time.sleep(self.plugin.testRunTimeout)
            client.close()

        t = threading.Thread(target=fake_client, args=(listener.address,))
        t.start()
        conn = self.plugin._acceptConns(listener)
        self.assertTrue(hasattr(conn, "send"))
        self.assertTrue(hasattr(conn, "recv"))
        t.join()


class TestProcserver(FunctionalTestCase):
    def setUp(self):
        super(TestProcserver, self).setUp()
        self.session = session.Session()

    def test_dispatch_tests_receive_events(self):
        ssn = {
            "config": self.session.config,
            "verbosity": 1,
            "startDir": support_file("scenario/tests_in_package"),
            "topLevelDir": support_file("scenario/tests_in_package"),
            "logLevel": 100,
            "pluginClasses": [
                discovery.DiscoveryLoader,
                testcases.TestCaseLoader,
                buffer.OutputBufferPlugin,
            ],
        }
        conn = Conn(
            [
                "pkg1.test.test_things.SomeTests.test_ok",
                "pkg1.test.test_things.SomeTests.test_failed",
            ]
        )
        procserver(ssn, conn)

        # check conn calls
        expect = [
            (
                "pkg1.test.test_things.SomeTests.test_ok",
                [
                    ("startTest", {}),
                    ("setTestOutcome", {"outcome": "passed"}),
                    ("testOutcome", {"outcome": "passed"}),
                    ("stopTest", {}),
                ],
            ),
            (
                "pkg1.test.test_things.SomeTests.test_failed",
                [
                    ("startTest", {}),
                    (
                        "setTestOutcome",
                        {
                            "outcome": "failed",
                            "expected": False,
                            "metadata": {
                                "stdout": """\
-------------------- >> begin captured stdout << ---------------------
Hello stdout

--------------------- >> end captured stdout << ----------------------"""
                            },
                        },
                    ),
                    (
                        "testOutcome",
                        {
                            "outcome": "failed",
                            "expected": False,
                            "metadata": {
                                "stdout": """\
-------------------- >> begin captured stdout << ---------------------
Hello stdout

--------------------- >> end captured stdout << ----------------------"""
                            },
                        },
                    ),
                    ("stopTest", {}),
                ],
            ),
        ]
        for val in conn.sent:
            if val is None:
                break
            test, events = val
            exp_test, exp_events = expect.pop(0)
            self.assertEqual(test, exp_test)
            for method, event in events:
                exp_meth, exp_attr = exp_events.pop(0)
                self.assertEqual(method, exp_meth)
                for attr, val in exp_attr.items():
                    self.assertEqual(getattr(event, attr), val)


class MPPluginTestRuns(FunctionalTestCase):
    def test_tests_in_package(self):
        proc = self.runIn(
            "scenario/tests_in_package", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 25 tests")
        self.assertEqual(proc.poll(), 1)

    def test_package_in_lib(self):
        proc = self.runIn(
            "scenario/package_in_lib", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 3 tests")
        self.assertEqual(proc.poll(), 1)

    def test_module_fixtures(self):
        proc = self.runIn(
            "scenario/module_fixtures", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)

    def test_class_fixtures(self):
        proc = self.runIn(
            "scenario/class_fixtures", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 7 tests")
        self.assertEqual(proc.poll(), 0)

    def test_large_number_of_tests_stresstest(self):
        proc = self.runIn(
            "scenario/many_tests",
            "-v",
            "--plugin=nose2.plugins.mp",
            "--plugin=nose2.plugins.loader.generators",
            "-N=1",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 600 tests")
        self.assertEqual(proc.poll(), 0)

    def test_socket_stresstest(self):
        proc = self.runIn(
            "scenario/many_tests_socket",
            "-v",
            "-c scenario/many_test_socket/nose2.cfg",
            "--plugin=nose2.plugins.mp",
            "--plugin=nose2.plugins.loader.generators",
            "-N=1",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 600 tests")
        self.assertEqual(proc.poll(), 0)

    def test_too_many_procs(self):
        # Just need to run the mp plugin with less tests than
        # processes.
        proc = self.runModuleAsMain(
            "scenario/one_test/tests.py",
            "--log-level=debug",
            "--plugin=nose2.plugins.mp",
            "-N=2",
        )
        ret_vals = six.moves.queue.Queue()

        def save_return():
            """
            Popen.communciate() blocks.  Use a thread-safe queue
            to return any exceptions.  Ideally, this completes
            and returns None.
            """
            try:
                self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
                self.assertEqual(proc.poll(), 0)
                ret_vals.put(None)
            except Exception as exc:
                ret_vals.put(exc)

        thread = threading.Thread(target=save_return)
        thread.start()

        # 1 minute should be more than sufficient for this
        # little test case.
        try:
            exc = ret_vals.get(True, 60)
        except six.moves.queue.Empty:
            exc = "MP Test timed out"
            proc.kill()
        self.assertIsNone(exc, str(exc))

    def test_with_output_buffer(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "--plugin=nose2.plugins.buffer",
            "-N=2",
            "-B",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)

    def test_unknown_module(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "-B",
            "does.not.exists.module",
            "does.not.exists.module2",
        )
        expected_results = (
            r"does\.not\.exists\.module2 (\S+) \.\.\. ERROR\n"
            r"does\.not\.exists\.module (\S+) \.\.\. ERROR"
        )
        self.assertTestRunOutputMatches(proc, stderr=expected_results)
        self.assertEqual(proc.poll(), 1)


class MPTestClassSupport(FunctionalTestCase):
    def test_testclass_discover(self):
        proc = self.runIn(
            "scenario/test_classes_mp", "-v", "--plugin=nose2.plugins.mp", "-N=2"
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 13 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_by_module(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_by_class(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp.Test",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 8 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_parameters(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp.Test.test_params",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_generators(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_classes_mp.Test.test_gen",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)


class MPClassFixturesSupport(FunctionalTestCase):
    def test_testcase_class_fixtures(self):
        proc = self.runIn(
            "scenario/class_fixtures", "-v", "test_cf_testcase.Test.test_1"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_class_fixtures_mp(self):
        proc = self.runIn(
            "scenario/class_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_cf_testcase.Test.test_1",
        )
        # XXX mp plugin runs the entire class if a class fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_class_fixtures_report_mp(self):
        proc = self.runIn(
            "scenario/class_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_cf_testcase.Test.test_1",
        )
        # report should show correct names for all tests
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_1 \(test_cf_testcase.Test"
            + _method_name("test_1")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_2 \(test_cf_testcase.Test"
            + _method_name("test_2")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_parameters(self):
        proc = self.runIn(
            "scenario/test_classes_mp", "-v", "test_fixtures_mp.Test.test_params"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_parameters_mp(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_fixtures_mp.Test.test_params",
        )
        # XXX mp plugin runs the entire class if a class fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_generators(self):
        proc = self.runIn(
            "scenario/test_classes_mp", "-v", "test_fixtures_mp.Test.test_gen"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testclass_class_fixtures_and_generators_mp(self):
        proc = self.runIn(
            "scenario/test_classes_mp",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_fixtures_mp.Test.test_gen",
        )
        # XXX mp plugin runs the entire class if a class fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 5 tests")
        self.assertEqual(proc.poll(), 0)


class MPModuleFixturesSupport(FunctionalTestCase):
    def test_testcase_module_fixtures(self):
        proc = self.runIn(
            "scenario/module_fixtures", "-v", "test_mf_testcase.Test.test_1"
        )
        # main process runs selected tests
        self.assertTestRunOutputMatches(proc, stderr="Ran 1 test")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_module_fixtures_mp(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_mf_testcase.Test.test_1",
        )
        # XXX mp plugin runs the entire module if a module fixture is detected
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)

    def test_testcase_module_fixtures_report_mp(self):
        proc = self.runIn(
            "scenario/module_fixtures",
            "-v",
            "--plugin=nose2.plugins.mp",
            "-N=2",
            "test_mf_testcase.Test.test_1",
        )
        # report should show correct names for all tests
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_1 \(test_mf_testcase.Test"
            + _method_name("test_1")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(
            proc,
            stderr=r"test_2 \(test_mf_testcase.Test"
            + _method_name("test_2")
            + r"\) ... ok",
        )
        self.assertTestRunOutputMatches(proc, stderr="Ran 2 tests")
        self.assertEqual(proc.poll(), 0)



import os
import re

from setuptools import find_packages, setup


def read_version(filename):
    # use of " over ' will be enforced by "black"
    version_pattern = re.compile(r'__version__ = "([^"]*)"')
    with open(filename) as fp:
        for line in fp:
            m = version_pattern.match(line)
            if m:
                return m.group(1)
    raise Exception("could not parse version from {}".format(filename))


MAINTAINER = "Stephen Rosen"
MAINTAINER_EMAIL = "dev@nose2.io"

LONG_DESCRIPTION = open(os.path.join(os.path.dirname(__file__), "README.rst")).read()

setup(
    name="nose2",
    version=read_version("nose2/__init__.py"),
    packages=find_packages(),
    extras_require={
        "coverage_plugin": ["coverage"],
        "dev": [
            "Sphinx",
            "sphinx_rtd_theme",
            "mock",
            "sphinx-issues",
        ],
    },
    entry_points={"console_scripts": ["nose2 = nose2:discover"]},
    # descriptive package info below
    description="unittest2 with plugins, the successor to nose",
    long_description=LONG_DESCRIPTION,
    maintainer=MAINTAINER,
    maintainer_email=MAINTAINER_EMAIL,
    url="https://github.com/nose-devs/nose2",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
    ],
    keywords=["unittest", "testing", "tests"],
)
